package cn.ibizlab.core.data.lite;

import com.alibaba.fastjson.JSON;
import net.ibizsys.model.IPSSystem;
import net.ibizsys.model.PSModelServiceImpl;
import net.ibizsys.model.dataentity.IPSDataEntity;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class LiteStorage {
    private static LiteModelFeignClient service;

    public static void setLiteModelService(LiteModelFeignClient service)
    {
        if(LiteStorage.service==null)
            LiteStorage.service=service;
    }

    public static LiteModelFeignClient getLiteModelService()
    {
        return service;
    }

    public static EntityModel getEntityModel(String system,String entity)
    {

        return service.getProxyEntityModel(system,entity);
    }


    @Autowired
    private LiteModelFeignClient client;

    @PostConstruct
    public void init(){
        LiteStorage.setLiteModelService(client);
    }



    @Value("${ibiz.model.path:/app/file/model/}")
    private  String modelPath;

    private  Map<String, IPSSystem> dynamicSystems;
    public  synchronized IPSSystem getDynamicSystem(String system)
    {
        if(dynamicSystems==null)
            dynamicSystems=new HashMap<>();
        if(dynamicSystems.containsKey(system))
            return dynamicSystems.get(system);

        return initDynamicSystem(system);
    }

    public  synchronized IPSSystem initDynamicSystem(String system)
    {
        if(dynamicSystems==null)
            dynamicSystems=new HashMap<>();

        PSModelServiceImpl psModelService = new PSModelServiceImpl();
        psModelService.setPSModelFolderPath(modelPath+ File.separator+system+File.separator+"model");
        IPSSystem iPSSystem = null;
        try {
            iPSSystem = psModelService.getPSSystem();
            dynamicSystems.put(system,iPSSystem);

        } catch (Exception e) {
            throw new RuntimeException(String.format("加载系统模型错误:%s", e.getMessage()));
        }
        return iPSSystem;
    }

    public  synchronized EntityModel getDynamicEntity(String system,String entity) throws Exception
    {
        IPSSystem iPSSystem=getDynamicSystem(system);
        if(iPSSystem==null)
            return null;
        IPSDataEntity dataEntity = iPSSystem.getPSDataEntity(entity,true);
        if(dataEntity==null)
            return null;
        EntityModel entityModel=new EntityModel();
        MetaEntityModel metaEntityModel=new MetaEntityModel();
        metaEntityModel.setEntityId(dataEntity.getId()).setEntityName(dataEntity.getName())
                .setLogicName(dataEntity.getLogicName()).setCodeName(dataEntity.getCodeName()).setTableName(dataEntity.getTableName())
                .setSystemId(system).setSystemName(iPSSystem.getLogicName()).setShowOrder(0);

        if(dataEntity.isLogicValid())
        {
            List<Setting> settingList=new ArrayList<>();
            String val=dataEntity.getValidLogicValue();
            if(StringUtils.isEmpty(val))val="1";
            String deVal=dataEntity.getInvalidLogicValue();
            if(StringUtils.isEmpty(deVal))deVal="0";
            settingList.add(new Setting().setProperty("logicval").setValue(val));
            settingList.add(new Setting().setProperty("logicdelval").setValue(deVal));
            metaEntityModel.setExtParams(JSON.toJSONString(settingList));
        }
        entityModel.setEntity(metaEntityModel);
        for(IPSDEField defield:dataEntity.getAllPSDEFields())
        {

        }

        return entityModel;
    }
}
