package cn.ibizlab.core.data.model;

import cn.ibizlab.core.data.dto.BaseData;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.apache.ibatis.mapping.VendorDatabaseIdProvider;
import org.springframework.util.StringUtils;

import java.util.*;

@Getter
@Setter
@NoArgsConstructor
@Accessors(chain = true)
@JsonInclude(JsonInclude.Include.NON_EMPTY)
@JsonIgnoreProperties(ignoreUnknown = true)
public class POSchema {

    private String name;

    private String remarks;

    private String dsType;

    private String defaultDataSource;

    private String logicVal;
    private String logicDelVal;

    public String getLogicVal() {
        if(StringUtils.isEmpty(logicVal))
            return "1";
        return logicVal;
    }

    public String getLogicDelVal() {
        if(StringUtils.isEmpty(logicDelVal))
            return "0";
        return logicDelVal;
    }

    private List<Column> columns;

    private List<Column> transients;

    public POSchema setColumns(List<Column> columns)
    {
        this.columns=columns;
        refreshColumnMaps();
        return this;
    }

    public POSchema setTransients(List<Column> transients)
    {
        this.transients=transients;
        refreshColumnMaps();
        return this;
    }

    public POSchema addColumn(Column column)
    {
        if(columns==null)
            columns=new ArrayList<>();
        columns.add(column);
        refreshColumn(column);
        return this;
    }

    public POSchema addTransient(Column column)
    {
        if(transients==null)
            transients=new ArrayList<>();
        transients.add(column);
        refreshColumn(column);
        return this;
    }

    private List<ForeignKeyConstraint> foreignKeyConstraints;

    public POSchema addForeignKeyConstraint(ForeignKeyConstraint foreignKeyConstraint)
    {
        if(foreignKeyConstraints==null)
            foreignKeyConstraints=new ArrayList<>();
        foreignKeyConstraints.add(foreignKeyConstraint);

        return this;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String,Column> columnMaps;


    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String,String> baseColumnMap;


    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String,String> resultMap;

    @JsonIgnore
    @JSONField(serialize = false)
    private Map<String,String> keyMap;

    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String, Column> getColumnMaps() {
        if(columns!=null&&columnMaps==null)
        {
            refreshColumnMaps();
        }
        return columnMaps;
    }

    public boolean needTrans = false;

    public void refreshColumnMaps()
    {
        columnMaps=new LinkedHashMap<>();
        resultMap=new LinkedHashMap<>();
        baseColumnMap=new LinkedHashMap<>();
        keyMap=new LinkedHashMap<>();
        columns.forEach(column -> {
            refreshColumn(column);
        });
        if(logicValid&&logicValidColumn==null)
            logicValid=false;
    }

    public void refreshColumn(Column column)
    {
        if(columnMaps==null)
            columnMaps=new LinkedHashMap<>();
        if(resultMap==null)
            resultMap=new LinkedHashMap<>();
        if(baseColumnMap==null)
            baseColumnMap=new LinkedHashMap<>();
        if(keyMap==null)
            keyMap=new LinkedHashMap<>();

        columnMaps.put(column.getName().toLowerCase(),column);

        if(!column.isComputed())
        {
            if(column.isLogicValid()) {
                this.setLogicValid(true);
                this.setLogicValidColumn(column);
                if(StringUtils.isEmpty(column.getDefaultValue()))
                    column.setDefaultValue(this.getLogicVal());

                logicValidCond  =  " "+column.getName()+"=";
                logicValidDelCond = " "+column.getName()+"=";
                if(column.isText()) {
                    logicValidCond += ("'" + this.getLogicVal() + "' ");
                    logicValidDelCond += ("'" + this.getLogicDelVal() + "' ");
                }
                else {
                    logicValidCond += (this.getLogicVal() + " ");
                    logicValidDelCond += (this.getLogicDelVal() + " ");
                }
            }
            if(column.isTenant())
                this.setTenantColumn(column);
            if(column.isCreateTime())
                this.setCreateTimeColumn(column);
            if(column.isLastModify())
                this.setLastModifyColumn(column);
        }
        if((!StringUtils.isEmpty(column.getAlias()))&&column.getAlias().equalsIgnoreCase(column.getName()))
        {
            needTrans=true;
            columnMaps.put(column.getAlias().toLowerCase(), column);
            resultMap.put(column.getName().toLowerCase(),column.getAlias().toLowerCase());
            if(!column.isComputed())
            {
                baseColumnMap.put(column.getName().toLowerCase(),column.getAlias().toLowerCase());
                if(column.isPrimaryKey())
                    keyMap.put(column.getName().toLowerCase(),column.getAlias().toLowerCase());
            }

        }
        else
        {
            resultMap.put(column.getName().toLowerCase(), column.getName().toLowerCase());
            if(!column.isComputed())
            {
                baseColumnMap.put(column.getName().toLowerCase(), column.getName().toLowerCase());
                if(column.isPrimaryKey())
                    keyMap.put(column.getName().toLowerCase(),column.getName().toLowerCase());
            }
        }

    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String, String> getResultMap() {
        if(columns!=null&&resultMap==null)
        {
            refreshColumnMaps();
        }
        return resultMap;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String, String> getBaseColumnMap() {
        if(columns!=null&&baseColumnMap==null)
        {
            refreshColumnMaps();
        }
        return baseColumnMap;
    }



    @JsonIgnore
    @JSONField(serialize = false)
    public Map<String, String> getKeyMap() {
        if(columns!=null&&keyMap==null)
        {
            refreshColumnMaps();
        }
        return keyMap;
    }

    public Column getColumn(String name)
    {
        if(getColumnMaps()!=null)
        {
            return columnMaps.get(name.toLowerCase());
        }
        return null;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Column lastModifyColumn;


    @JsonIgnore
    @JSONField(serialize = false)
    private Column createTimeColumn;


    @JsonIgnore
    @JSONField(serialize = false)
    private Column tenantColumn;


    @JsonIgnore
    @JSONField(serialize = false)
    private boolean logicValid=false;
    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isLogicValid()
    {
        if(logicValid&&logicValidColumn==null) {
            if (columns != null) {
                for (Column col:columns) {
                    if (col.isLogicValid()) {
                        logicValidColumn = col;
                        return logicValid;
                    }
                }
            }
            logicValid = false;
        }
        return logicValid;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Column logicValidColumn;


    @Getter
    @Setter
    @NoArgsConstructor
    @Accessors(chain = true)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    @JsonIgnoreProperties(ignoreUnknown = true)
    public static class Column{
        private String name;
        private String remarks;
        private String type;
        private Integer length;
        private Integer precision;
        private String defaultValue;
        private Boolean autoIncrement;
        private Boolean computed;
        private String alias;
        private String predefined;


        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isComputed()
        {
            return this.getComputed()!=null&&this.getComputed();
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isAutoIncrement()
        {
            return this.getAutoIncrement()!=null&&this.getAutoIncrement();
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isNullable()
        {
            return this.getConstraints()!=null&&this.getConstraints().isNullable();
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isPrimaryKey()
        {
            return this.getConstraints()!=null&&this.getConstraints().isPrimaryKey();
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isLogicValid()
        {
            return "LOGICVALID".equals(this.getPredefined());
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isLastModify()
        {
            return "UPDATEDATE".equals(this.getPredefined());
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isCreateTime()
        {
            return "CREATEDATE".equals(this.getPredefined());
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isTenant()
        {
            return "TENANT".equals(this.getPredefined());
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isText()
        {
            String type=this.getType().toUpperCase();
            return type.indexOf("TEXT")>=0||type.indexOf("CHAR")>=0||type.indexOf("LOB")>=0;
        }
        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isNumber()
        {
            String type=this.getType().toUpperCase();
            return type.indexOf("NUM")>=0||type.indexOf("FLOAT")>=0||type.indexOf("DOUBLE")>=0||type.indexOf("DECIMAL")>=0;
        }
        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isInt()
        {
            String type=this.getType().toUpperCase();
            return type.indexOf("INT")==0;
        }

        private Constraints constraints;

        public Constraints getConstraints(boolean createWhenNotExist)
        {
            if(createWhenNotExist)
            {
                if(constraints==null)
                    constraints=new Constraints();
            }
            return constraints;
        }

    }

    @Getter
    @Setter
    @NoArgsConstructor
    @Accessors(chain = true)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    @JsonIgnoreProperties(ignoreUnknown = true)
    public static class Constraints{
        private Boolean nullable;
        private Boolean primaryKey;
        private String primaryKeyName;
        private String foreignKeyName;
        private String referencedTableName;
        private String referencedColumnNames;

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isNullable()
        {
            return this.getNullable()!=null&&this.getNullable();
        }

        @JsonIgnore
        @JSONField(serialize = false)
        public boolean isPrimaryKey()
        {
            return this.getPrimaryKey()!=null&&this.getPrimaryKey();
        }

    }

    @Getter
    @Setter
    @NoArgsConstructor
    @Accessors(chain = true)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    @JsonIgnoreProperties(ignoreUnknown = true)
    public static class ForeignKeyConstraint{
        private String baseColumnNames;
        private String baseTableName;
        private String constraintName;
        private String referencedTableName;
        private String referencedColumnNames;
        private Boolean validate;
        private String onUpdate;
        private String onDelete;

    }

    @Getter
    @Setter
    @NoArgsConstructor
    @Accessors(chain = true)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    @JsonIgnoreProperties(ignoreUnknown = true)
    public static class Script{
        private String name;
        private String vendorProvider;
        private String declare;
        private String body;
        private Map params;

    }


    public List<Script> scripts;

    private static Map<String, String> provider = new HashMap<String, String>(){{
        put("oracle", "oracle");
        put("mysql", "mysql");
        put("postgresql", "postgresql");
        put("mppdb", "postgresql");
        put("dm", "oracle");
        put("dameng", "oracle");
        put("gbase", "oracle");
        put("h2", "mysql");
    }};



    public Script getScript(String name,String type)
    {
        if(scripts!=null)
        {
            String vendorProvider="mysql";
            if((!StringUtils.isEmpty(type))&&provider.containsKey(type.toLowerCase()))
                vendorProvider=provider.get(type.toLowerCase());
            for(Script script:scripts)
            {
                if(name.equalsIgnoreCase(script.getName())&&((script.getVendorProvider().indexOf(vendorProvider)>=0)||StringUtils.isEmpty(script.getVendorProvider())))
                    return script;
            }
        }
        else
		{
			scripts=new ArrayList<>();
		}
        return getDefaultQueryScript();
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public Script defaultQueryScript;

    @JsonIgnore
    @JSONField(serialize = false)
    public Script getDefaultQueryScript()
    {
        if(defaultQueryScript==null)
        {
            defaultQueryScript=new Script();
            defaultQueryScript.setName("default");
            defaultQueryScript.setVendorProvider("mysql,oracle,postgresql");
            String sql="select ";

            String cols="";
            if(getBaseColumnMap()!=null)
            {
                for(String key:baseColumnMap.keySet())
                {
                    if(!StringUtils.isEmpty(cols))
                        cols+=",";
                    cols+=("t."+key+" as "+baseColumnMap.get(key));
                }
            }
            else
                cols="*";

            sql += (cols+" from "+name+" t ");

            if(isLogicValid())
            {

                sql += " where t."+this.getLogicValidCond();

            }
            defaultQueryScript.setBody(sql);
        }
        return defaultQueryScript;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public String logicValidCond;
    @JsonIgnore
    @JSONField(serialize = false)
    public String logicValidDelCond;





    @JSONField(serialize = false)
    @JsonIgnore
    private boolean built = false;

    @JSONField(serialize = false)
    @JsonIgnore
    public synchronized POSchema build()
    {
        if(!built)
        {
            refreshColumnMaps();
            built=true;
        }
        return this;
    }

    public BaseData trans(BaseData source)
    {
        if(!needTrans)
            return source;
        else {
            BaseData target = new BaseData();
            for (Map.Entry<String, String> entry : resultMap.entrySet()) {
                if(source.keySet().contains(entry.getKey()))
                    target.set(entry.getValue(),source.get(entry.getKey()));
            }
            return target;
        }
    }

}
